# Install the base components of a 4.x cassandra server
# Configure all the instances declared in cassandra::instances property
#
# Look at profile::cassandra:node for more information about the
# instance(s) configuration
class profile::cassandra {

  include profile::prometheus::jmx
  $jmx_exporter_version = lookup('prometheus::jmx::version')

  $cassandra_user = 'cassandra'
  $cassandra_group = 'cassandra'
  $cassandra_home = '/home/cassandra'

  $cassandra_version = lookup('cassandra::version')
  $cassandra_archive_name = "apache-cassandra-${cassandra_version}-bin.tar.gz"
  $cassandra_bin_url = "https://dlcdn.apache.org/cassandra/${cassandra_version}/${cassandra_archive_name}"
  $cassandra_bin_checksum_type = lookup('cassandra::version_checksum_type')
  $cassandra_bin_checksum = lookup('cassandra::version_checksum')

  $systemd_service = 'cassandra@.service'

  $download_path = "/opt/${cassandra_archive_name}"

  $cassandra_install_directory = "/opt/cassandra-${cassandra_version}"

  $cassandra_base_data_directory = lookup('cassandra::base_data_directory')
  $cassandra_config_directory = lookup('cassandra::base_config_directory')
  $cassandra_log_directory = lookup('cassandra::base_log_directory')

  $cassandra_nodes = lookup('cassandra::nodes')
  $node_definition = $cassandra_nodes["$::fqdn"]
  $instances = $node_definition['instances']

  $default_instance_config = lookup('cassandra::default_instance_configuration')
  $clusters_config = lookup('cassandra::clusters')

  $jmx_remote = $default_instance_config['jmx_remote']
  $jmx_access_file = "${cassandra_config_directory}/jmxremote.access"
  $jmx_password_file = "${cassandra_config_directory}/jmxremote.password"

  $max_map_count = lookup("cassandra::max_map_count")

  group {$cassandra_group:
    system => true,
  }

  user {$cassandra_user:
    system => true,
    gid    => $cassandra_group,
    shell  => '/usr/sbin/nologin',
    home   => $cassandra_home,
  }

  file { [
      $cassandra_install_directory,
      $cassandra_config_directory,
    ]:
    ensure => directory,
    owner  => 'root',
    group  => 'root',
    mode   => '0755'
  }

  file { [
      $cassandra_base_data_directory,
      $cassandra_log_directory,
    ]:
    ensure => directory,
    owner  => $cassandra_user,
    group  => $cassandra_group,
    mode   => '0755'
  }

  ensure_packages(['openjdk-11-jdk', 'libnetty-java'])

  archive { 'cassandra':
    path            => $download_path,
    extract         => true,
    extract_command => 'tar xzf %s --strip-components=1 --no-same-owner --no-same-permissions',
    source          => $cassandra_bin_url,
    extract_path    => $cassandra_install_directory,
    checksum_type   => $cassandra_bin_checksum_type,
    checksum        => $cassandra_bin_checksum,
    creates         => "${cassandra_install_directory}/bin/cassandra",
    cleanup         => true,
    user            => 'root',
    group           => 'root',
    require         => File[$cassandra_install_directory],
  }
  -> file {'/opt/cassandra':
    ensure => link,
    force  => true,
    target => $cassandra_install_directory
  }

  ::systemd::unit_file {$systemd_service:
      ensure  => present,
      content => template('profile/cassandra/cassandra.service.erb'),
  }

  $config_files_to_copy = [
    'jvm11-clients.options',
    'jvm-clients.options',
    'logback-tools.xml',
  ]

  $config_files_to_copy.each | $file_name | {
    file { "${cassandra_config_directory}/${file_name}":
      ensure  => present,
      owner   => 'root',
      group   => 'root',
      mode    => '0644',
      source  => "/opt/cassandra/conf/${file_name}",
      require => [File[$cassandra_config_directory],
        Archive['cassandra']],
    }
  }

  file {"${cassandra_config_directory}/jmx_exporter.yml":
    ensure  => present,
    owner   => 'root',
    group   => 'root',
    mode    => '0644',
    source  => "https://raw.githubusercontent.com/prometheus/jmx_exporter/parent-${jmx_exporter_version}/example_configs/cassandra.yml",
    require => [File[$cassandra_config_directory]],
  }

  if $jmx_remote == true {
    $jmx_user = $default_instance_config['jmx_user']
    $jmx_password = $default_instance_config['jmx_password']

    file {$jmx_access_file:
      ensure  => present,
      owner   => 'root',
      group   => $cassandra_group,
      mode    => '0540',
      content => template('profile/cassandra/jmxremote.access.erb'),
      require => [File[$cassandra_config_directory]]
    }
    file {$jmx_password_file:
      ensure  => present,
      owner   => 'root',
      group   => $cassandra_group,
      mode    => '0540',
      content => template('profile/cassandra/jmxremote.password.erb'),
      require => [File[$cassandra_config_directory]]
    }
  } else {
    file {[ $jmx_access_file,
            $jmx_password_file ]:
      ensure => absent,
    }
  }

  sysctl { 'vm.max_map_count': value => $max_map_count }

  $instances.each | $instance_name, $instance_config | {
    $merged_instance_config = $default_instance_config + $instance_config
    $cluster_config = $clusters_config[$merged_instance_config["cluster_name"]]
    $merged_config = $cluster_config + $merged_instance_config

    profile::cassandra::instance{$instance_name:
      config => $merged_config
    }
  }

}

